__app_name__ = "clarc"
__version__ = "0.1.0"
__DB_NAME__ = "file::memory:?cache=shared"

(
    SUCCESS,
    DB_ERROR,
    OPERATIONAL_ERROR,
    FETCH_ERROR,
    STASH_ERROR,
) = range(5)

ERRORS = {
    DB_ERROR: "db connection error",
    FETCH_ERROR: "value fetch error",
    STASH_ERROR: "value stash error",
    OPERATIONAL_ERROR: "db operational error",
}


from typer.testing import CliRunner
from clarc import __app_name__, __version__, cli

runner = CliRunner()


def test_version():
    result = runner.invoke(cli.app, ["--version"])
    assert result.exit_code == 0
    assert f"{__app_name__} v{__version__}\n" in result.stdout


def test_init():
    result = runner.invoke(cli.app, ["init"])
    assert result.exit_code == 0
    assert f"successfully!" in result.stdout



import sqlite3
from typing import Optional
import typer
from clarc import ERRORS, __app_name__, __version__, clarc, __DB_NAME__
import json

app = typer.Typer()


def get_operations() -> clarc.Operations:
    conn = sqlite3.connect(__DB_NAME__)
    if clarc.DatabaseHandler(conn).is_db():
        return clarc.Operations(conn)
    else:
        typer.secho(
            'database not found. Please, run "clarc init"',
            fg=typer.colors.RED,
        )
        raise typer.Exit(1)


@app.command()
def init() -> None:
    """initialize the database."""
    app_init_error = get_operations().init_app()
    if app_init_error:
        typer.secho(
            f'database creation failed with "{ERRORS[app_init_error]}"',
            fg=typer.colors.RED,
        )
        raise typer.Exit(1)
    else:
        typer.secho(f"database is created successfully!", fg=typer.colors.GREEN)


@app.command()
def fetch(key: str = None, all: bool = False) -> None:
    """fetch the archived entries '--all' or by '--key' ${key}"""
    if all:
        fetch_list = get_operations().fetch_all()
        if isinstance(fetch_list, int):
            typer.secho(
                f'execution failed with {ERRORS[fetch_list]}',
                fg=typer.colors.RED,
            )
            raise typer.Exit(1)
        if len(fetch_list) == 0:
            typer.secho(
                "no entries in db yet", fg=typer.colors.YELLOW
            )
            raise typer.Exit()
        data = _response(fetch_list)
        typer.secho(
                data,
                fg=typer.colors.BLUE,
        )
    if key is not None:
        fetch_key = get_operations().fetch_key(key)
        if len(fetch_key) == 0:
            typer.secho(
                "no entries found for key search: " + key, fg=typer.colors.YELLOW
            )
            raise typer.Exit()
        data = _response(fetch_key)
        typer.secho(
            data,
            fg=typer.colors.BLUE,
        )


@app.command()
def upsert(
        key: str = typer.Argument(...),
        value: str = typer.Argument(...)
) -> None:
    """update on duplicate key else insert a new entry to db """
    get_operations().upsert(key, value)
    typer.secho("entry is successfully archived with key: " + key, fg=typer.colors.GREEN, bold=True)


@app.command()
def remove(
        key: str = typer.Argument(...)
) -> None:
    """remove an archived entry by specefic --key' ${key}"""
    fetch_key = get_operations().fetch_key_strict(key)
    if len(fetch_key) == 0:
        typer.secho(
            "no entries found for key search: " + key, fg=typer.colors.YELLOW
        )
        raise typer.Exit()
    get_operations().delete(key)
    typer.secho("entry is removed for key: " + key, fg=typer.colors.GREEN, bold=True)


def _version_callback(value: bool) -> None:
    if value:
        typer.echo(f"{__app_name__} v{__version__}")
        raise typer.Exit()


def _response(results):
    res_dict = {"total": len(results),
                "archives": []}
    for index, element in enumerate(results):
        arch = {"id": index, "key": element[0], "value": element[1]}
        res_dict["archives"].append(arch)
    return json.dumps(res_dict, indent=4)


@app.callback()
def main(
        version: Optional[bool] = typer.Option(
            None,
            "--version",
            "-v",
            help="Show the application's version and exit.",
            callback=_version_callback,
            is_eager=True,
        )
) -> None:
    return



from clarc import OPERATIONAL_ERROR
import sqlite3
from clarc import DB_ERROR, SUCCESS

CREATE_TABLE = '''CREATE TABLE  IF NOT EXISTS CLARC(
                           KEY CHAR(50) NOT NULL PRIMARY KEY,
                           VALUE CHAR(255) NOT NULL,
                           CREATED TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
                           UPDATED TIMESTAMP DEFAULT CURRENT_TIMESTAMP
                        );'''
FETCH_ALL = "SELECT * FROM CLARC LIMIT 10"
FETCH_BY_KEY = "SELECT * FROM CLARC WHERE KEY LIKE ?"
FETCH_BY_KEY_STRICT = "SELECT * FROM CLARC WHERE KEY= ?"
UPSERT_VALUE = "INSERT INTO CLARC (KEY, VALUE) VALUES(?, ?) ON CONFLICT(KEY) DO UPDATE SET VALUE= ?"
DELETE_VALUE = "DELETE FROM CLARC WHERE KEY= ?"


class DatabaseHandler:

    def __init__(self, db: sqlite3.Connection):
        self.db = db

    def is_db(self):
        try:
            self.db.cursor()
            return True
        except sqlite3.Error:
            return False

    def create_database(self) -> int:
        try:
            with self.db:
                cursor = self.db.cursor()
                cursor.execute(CREATE_TABLE)
                self.db.commit()
                return SUCCESS
        except sqlite3.Error:
            return DB_ERROR

    def init_app(self) -> int:
        database_code = self.create_database()
        if database_code != SUCCESS:
            return database_code
        return SUCCESS

    def query(self, sql: str, q=None):
        try:
            with self.db:
                cur = self.db.cursor()
                if q is None:
                    cur.execute(sql)
                else:
                    cur.execute(sql, q)
                return cur.fetchall()
        except (sqlite3.OperationalError, sqlite3.IntegrityError) as e:
            print(e)
            return OPERATIONAL_ERROR


class Operations(DatabaseHandler):
    def __init__(self, db):
        super().__init__(db)

    def fetch_all(self):
        return super().query(FETCH_ALL)

    def fetch_key(self, key: str):
        return super().query(FETCH_BY_KEY, ('%' + key + '%',))

    def fetch_key_strict(self, key: str):
        return super().query(FETCH_BY_KEY_STRICT, (key,))

    def upsert(self, key: str, value: str):
        super().query(UPSERT_VALUE, (key, value, value))

    def delete(self, key: str):
        super().query(DELETE_VALUE, (key,))
